package animation;

import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.Timer;

/**
 *  Using a timer to do some animation
 * @author hajo
 */
public class AnimationPanel extends JPanel implements ActionListener {

    private final Timer timer;
    private final long threshold = 1000000;
    private final ImageIcon blurred;
    private boolean animate = true;
    private long startTime = 0;
    private int x = 0;
    private int y = 0;

    public AnimationPanel() {
        URL imgURL = getClass().getResource("/animation/dice6blurred.png");
        blurred = new ImageIcon(imgURL, "blurred");
        timer = new Timer(50, this);
        timer.setDelay(50);
        start();
    }

    // Automatically called by Swing
    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        if (animate) {
            blurred.paintIcon(this, g, x++, y);
            // Necessary for linux users to draw and animate image correctly
            Toolkit.getDefaultToolkit().sync();
            g.dispose();
        }
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        System.out.println("Got event from timer " + e.getWhen());
        if (startTime == 0) {
            startTime = e.getWhen();
        }
        if (e.getWhen() - startTime > threshold) {
            stop();
            startTime = 0;
        }
        repaint();
    }

    public void start() {
        animate = true;
        timer.start();
    }

    public void stop() {
        animate = false;
        timer.stop();
    }
}
