package towersofhanoi;

public class TowersOfHanoi {

    public static final int N_DISCS = 3;
    private final Rod a = new Rod("A", N_DISCS);
    private final Rod b = new Rod("B", N_DISCS);
    private final Rod c = new Rod("C", N_DISCS);

    public TowersOfHanoi() {
        for (int i = 1; i <= N_DISCS; i++) {
            a.push(new Disc(i));
        }
    }

    // The recursive method
    public void move(int n, Rod from, Rod to, Rod aux) {
        plot(n, "->");
        if (n > 0) {
            move(n - 1, from, aux, to);  // NOTE: Toggle to/aux
            to.push(from.pop());
            dump(n);                     // Visualize ... 
            move(n - 1, aux, to, from);  // NOTE: Toggle from/aux     
        }
        plot(n, "<-");
    }

    // ------------- Helper Methods -----------------
    
    public void run() {
        System.out.println("Towers of Hanoi");
        plot(N_DISCS, a);
        move(N_DISCS, a, b, c);
        System.out.println("Finished");
    }

    private void dump(int level) {
        plot(level, a);
        plot(level, b);
        plot(level, c);
    }

    private static void plot(int level, String s) {
        indent(N_DISCS - level);
        System.out.print("n = " + level + " ");
        System.out.println(s);
    }

    private static void plot(int level, Rod rod) {
        indent(N_DISCS - level);
        System.out.println(rod);
    }

    private static void indent(int n) {
        for (int i = 0; i < n; i++) {
            System.out.print("...");
        }
    }

    public static void main(String[] args) {
        new TowersOfHanoi().run();
    }

}
