package visualtowersofhanoi;

import java.util.*;

public class HanoiModel {

    private final int[][] piles;
    private final int[] size;
    private final LinkedList<Move> moves;
    private final int disks;

    private static class Move {

        public final int from, to;

        Move(int from, int to) {
            this.from = from;
            this.to = to;
        }
    }

    public HanoiModel(int n) {
        disks = n;
        piles = new int[3][n];
        size = new int[3];
        size[0] = n;
        for (int i = 0; i < n; i++) {
            piles[0][i] = n - i;
        }
        moves = new LinkedList<>();
        computeMoves(n, 0, 1, 2);
    }

    private void computeMoves(int size, int from, int to, int aux) {
        if (size == 0) {
            return;
        }
        computeMoves(size - 1, from, aux, to);
        moves.add(new Move(from, to));
        computeMoves(size - 1, aux, to, from);
    }

    public void moveOne() {
        if (!moves.isEmpty()) {
            Move mv = moves.remove(0);
            int disk = piles[mv.from][size[mv.from] - 1];
            piles[mv.to][size[mv.to]] = disk;
            size[mv.from]--;
            size[mv.to]++;
        }
    }

    public int getSize(int pile) {
        return size[pile];
    }

    public int getDisk(int pile, int pos) {
        return piles[pile][pos];
    }

    public int getDisks() {
        return disks;
    }
}
